#include <cstdio>
#include <set>
#include <string>
#include <utility>

#include "settings.hh"
#include "confparser.hh"

using std::string;
using std::set;
using std::pair;

unsigned BITRATE;
unsigned FPS;
double DELAY_MULTIPLIER;
string LAVCOPTS;
string WINDOW_NAME;
unsigned XOFFSET;
unsigned YOFFSET;

string OUTPUT_FILE;


namespace
{
    ConfParser config;
    
    static const char cfgfilename[] = "xgetimage.cfg";

    const ConfParser::Field& GetConf(const char *sect, const char *var)
    {
        try
        {
            //fprintf(stderr, "Trying '%s' '%s'\n", sect, var);
            return config[sect][var];
        }
        catch(ConfParser::invalid_section sect_error)
        {
            typedef string errortype;
            static set<errortype> displayed;
            
            const errortype ErrorName(sect_error.GetSection());
            
            if(displayed.find(ErrorName) == displayed.end())
            {
                fprintf(stderr,
                    "> %s: Missing section [%s]\n",
                    cfgfilename,
                    ErrorName.c_str()
                       );
                displayed.insert(ErrorName);
            }

            static const ConfParser::Field error;
            return error;
        }
        catch(ConfParser::invalid_field field_error)
        {
            typedef pair<string, string> errortype;
            static set<errortype> displayed;
            
            const errortype ErrorName(field_error.GetField().c_str(),
                                      field_error.GetSection().c_str());
            
            if(displayed.find(ErrorName) == displayed.end())
            {
                fprintf(stderr,
                    "> %s: Missing \"%s\" in [%s]\n",
                   cfgfilename,
                   ErrorName.first.c_str(),/* fieldname */
                   ErrorName.second.c_str()/* section name */
                       );

                displayed.insert(ErrorName);
            }

            static const ConfParser::Field error;
            return error;
        }
    }
}

class SettingsLoader
{
public:
    SettingsLoader()
    {
        FILE *fp = fopen(cfgfilename, "rt");
        if(!fp)
        {
            perror(cfgfilename);
            exit(-1);
        }
        config.Parse(fp);
        fclose(fp);
    
        BITRATE            = GetConf("rate", "BITRATE");
        FPS                = GetConf("rate", "FPS");
        DELAY_MULTIPLIER   = GetConf("rate", "DELAY");
        LAVCOPTS    = WstrToAsc(GetConf("rate", "LAVCOPTS"));
        
        WINDOW_NAME = WstrToAsc(GetConf("source", "WINDOW"));
        XOFFSET     = GetConf("source", "XOFFSET");
        YOFFSET     = GetConf("source", "YOFFSET");
        
        OUTPUT_FILE = WstrToAsc(GetConf("target", "OUTPUTFILE"));

        fprintf(stderr, "FPS=%u, DELAY=%g\n", FPS, DELAY_MULTIPLIER);
    }
};

static SettingsLoader sets;

string MEMORY_FILE = "/tmp/xgetimage.mem";
